@@ -21,10 +21,6 @@
 
 /* This file defines all string functions */
 
-#ifdef USE_PRAGMA_INTERFACE
-#pragma interface			/* gcc class implementation */
-#endif
-
 extern size_t username_char_length;
 
 class Item_str_func :public Item_func
@@ -109,10 +105,14 @@ public:
    :Item_str_ascii_func(thd, a) { }
   Item_str_ascii_checksum_func(THD *thd, Item *a, Item *b)
    :Item_str_ascii_func(thd, a, b) { }
-  bool eq(const Item *item, bool binary_cmp) const
+  Item_str_ascii_checksum_func(THD *thd, Item *a, Item *b, Item *c)
+   :Item_str_ascii_func(thd, a, b, c) { }
+  bool eq(const Item *item, const Eq_config &config) const override
   {
     // Always use binary argument comparison: MD5('x') != MD5('X')
-    return Item_func::eq(item, true);
+    Eq_config new_config= config;
+    new_config.binary_cmp= true;
+    return Item_func::eq(item, new_config);
   }
 };
 
@@ -133,13 +133,17 @@ public:
    :Item_str_func(thd, a, b, c) { }
   Item_str_binary_checksum_func(THD *thd, Item *a, Item *b, Item *c, Item *d)
    :Item_str_func(thd, a, b, c, d) { }
-  bool eq(const Item *item, bool binary_cmp) const
+  Item_str_binary_checksum_func(THD *thd, Item *a, Item *b, Item *c, Item *d, Item *e)
+   :Item_str_func(thd, a, b, c, d, e) { }
+  bool eq(const Item *item, const Eq_config &config) const override
   {
     /*
       Always use binary argument comparison:
         FROM_BASE64('test') != FROM_BASE64('TEST')
     */
-    return Item_func::eq(item, true);
+    Eq_config new_config= config;
+    new_config.binary_cmp= true;
+    return Item_func::eq(item, new_config);
   }
 };
 
@@ -159,7 +163,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("md5") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_md5>(thd, this); }
 };
 
@@ -175,7 +179,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("sha") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_sha>(thd, this); }
 };
 
@@ -191,7 +195,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("sha2") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_sha2>(thd, this); }
 };
 
@@ -208,7 +212,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("to_base64") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_to_base64>(thd, this); }
 };
 
@@ -225,7 +229,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("from_base64") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_from_base64>(thd, this); }
 };
 
@@ -272,7 +276,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("aes_encrypt") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_aes_encrypt>(thd, this); }
 };
 
@@ -291,10 +295,39 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("aes_decrypt") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_aes_decrypt>(thd, this); }
 };
 
+class Item_func_kdf :public Item_str_binary_checksum_func
+{
+  uint key_length;
+public:
+  Item_func_kdf(THD *thd, Item *a, Item *b)
+   : Item_str_binary_checksum_func(thd, a, b) {}
+  Item_func_kdf(THD *thd, Item *a, Item *b, Item *c)
+   : Item_str_binary_checksum_func(thd, a, b, c) {}
+  Item_func_kdf(THD *thd, Item *a, Item *b, Item *c, Item *d)
+   : Item_str_binary_checksum_func(thd, a, b, c, d) {}
+  Item_func_kdf(THD *thd, Item *a, Item *b, Item *c, Item *d, Item *e)
+   : Item_str_binary_checksum_func(thd, a, b, c, d, e) {}
+  bool fix_length_and_dec(THD *thd) override;
+  String *val_str(String *) override;
+  bool check_vcol_func_processor(void *arg) override
+  {
+    if (arg_count > 4)
+      return FALSE;
+    return mark_unsupported_function(func_name(), "()", arg, VCOL_SESSION_FUNC);
+  }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("kdf") };
+    return name;
+  }
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_kdf>(thd, this); }
+};
+
 class Item_func_natural_sort_key : public Item_str_func
 {
 public:
@@ -307,7 +340,7 @@ public:
     return name;
   }
   bool fix_length_and_dec(THD *thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   {
     return get_item_copy<Item_func_natural_sort_key>(thd, this);
   }
@@ -317,6 +350,8 @@ public:
 
 class Item_func_concat :public Item_str_func
 {
+  bool check_arguments() const override
+  { return check_argument_types_can_return_str(0, arg_count); }
 protected:
   String tmp_value;
   /*
@@ -344,7 +379,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("concat") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_concat>(thd, this); }
 };
 
@@ -355,6 +390,8 @@ public:
 */
 class Item_func_concat_operator_oracle :public Item_func_concat
 {
+  bool check_arguments() const override
+  { return check_argument_types_can_return_str(0, arg_count); }
 public:
   Item_func_concat_operator_oracle(THD *thd, List<Item> &list)
    :Item_func_concat(thd, list)
@@ -375,7 +412,7 @@ public:
       print_sql_mode_qualified_name(str, query_type);
     print_args_parenthesized(str, query_type);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   {
     return get_item_copy<Item_func_concat_operator_oracle>(thd, this);
   }
@@ -390,7 +427,7 @@ public:
   String *val_str(String *) override;
   bool fix_length_and_dec(THD *thd) override
   {
-    collation.set(system_charset_info);
+    collation.set(system_charset_info_for_i_s);
     max_length= MAX_BLOB_WIDTH;
     set_maybe_null();
     return FALSE;
@@ -400,12 +437,14 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("decode_histogram") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_decode_histogram>(thd, this); }
 };
 
 class Item_func_concat_ws :public Item_str_func
 {
+  bool check_arguments() const override
+  { return check_argument_types_can_return_str(0, arg_count); }
   String tmp_value;
 public:
   Item_func_concat_ws(THD *thd, List<Item> &list): Item_str_func(thd, list) {}
@@ -417,7 +456,7 @@ public:
     return name;
   }
   table_map not_null_tables() const override { return 0; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_concat_ws>(thd, this); }
 };
 
@@ -438,7 +477,7 @@ public:
   {
     return mark_unsupported_function(func_name(), "()", arg, VCOL_NON_DETERMINISTIC);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   {
     return get_item_copy<Item_func_random_bytes>(thd, this);
   }
@@ -458,7 +497,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("reverse") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_reverse>(thd, this); }
 };
 
@@ -484,7 +523,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("replace") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_replace>(thd, this); }
 };
 
@@ -509,7 +548,7 @@ public:
       print_sql_mode_qualified_name(str, query_type);
     print_args_parenthesized(str, query_type);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_replace_oracle>(thd, this); }
 };
 
@@ -549,7 +588,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("regexp_replace") };
     return name;
   }
-  Item *get_copy(THD *thd) override { return 0;}
+  Item *do_get_copy(THD *thd) const override { return 0;}
 };
 
 
@@ -559,14 +598,14 @@ public:
   Item_func_regexp_replace_oracle(THD *thd, Item *a, Item *b, Item *c)
    :Item_func_regexp_replace(thd, a, b, c)
   {}
-  const Schema *schema() const { return &oracle_schema_ref; }
-  bool fix_length_and_dec(THD *thd)
+  const Schema *schema() const override { return &oracle_schema_ref; }
+  bool fix_length_and_dec(THD *thd) override
   {
     bool rc= Item_func_regexp_replace::fix_length_and_dec(thd);
     set_maybe_null(); // Empty result is converted to NULL
     return rc;
   }
-  String *val_str(String *str)
+  String *val_str(String *str) override
   {
     return val_str_internal(str, true);
   }
@@ -594,7 +633,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("regexp_substr") };
     return name;
   }
-  Item *get_copy(THD *thd) override { return 0; }
+  Item *do_get_copy(THD *thd) const override { return 0; }
 };
 
 
@@ -612,7 +651,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("insert") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_insert>(thd, this); }
 };
 
@@ -639,7 +678,7 @@ public:
     return name;
   }
   bool fix_length_and_dec(THD *thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_lcase>(thd, this); }
 };
 
@@ -653,7 +692,7 @@ public:
     return name;
   }
   bool fix_length_and_dec(THD *thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_ucase>(thd, this); }
 };
 
@@ -666,12 +705,13 @@ public:
   bool hash_not_null(Hasher *hasher) override;
   String *val_str(String *) override;
   bool fix_length_and_dec(THD *thd) override;
+  enum Functype functype() const override { return LEFT_FUNC; }
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("left") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_left>(thd, this); }
 };
 
@@ -688,7 +728,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("right") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_right>(thd, this); }
 };
 
@@ -712,12 +752,13 @@ public:
     print_sql_mode_qualified_name(str, query_type);
     print_args_parenthesized(str, query_type);
   }
+  enum Functype functype() const override { return SUBSTR_FUNC; }
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("substr") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_substr>(thd, this); }
 };
 
@@ -734,7 +775,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("sformat") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_sformat>(thd, this); }
 };
 
@@ -770,7 +811,7 @@ public:
       print_sql_mode_qualified_name(str, query_type);
     print_args_parenthesized(str, query_type);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_substr_oracle>(thd, this); }
 };
 
@@ -787,7 +828,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("substring_index") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_substr_index>(thd, this); }
 
 };
@@ -830,7 +871,7 @@ public:
   }
   void print(String *str, enum_query_type query_type) override;
   virtual LEX_CSTRING mode_name() const { return { "both", 4}; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_trim>(thd, this); }
 };
 
@@ -851,7 +892,7 @@ public:
     set_maybe_null();
     return res;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_trim_oracle>(thd, this); }
 };
 
@@ -874,7 +915,7 @@ public:
   }
   LEX_CSTRING mode_name() const override
   { return { STRING_WITH_LEN("leading") }; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_ltrim>(thd, this); }
 };
 
@@ -895,7 +936,7 @@ public:
     set_maybe_null();
     return res;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_ltrim_oracle>(thd, this); }
 };
 
@@ -914,7 +955,7 @@ public:
   }
   LEX_CSTRING mode_name() const override
   { return { STRING_WITH_LEN("trailing") }; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_rtrim>(thd, this); }
 };
 
@@ -935,7 +976,7 @@ public:
     set_maybe_null();
     return res;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_rtrim_oracle>(thd, this); }
 };
 
@@ -978,7 +1019,7 @@ public:
   }
   static char *alloc(THD *thd, const char *password, size_t pass_len,
                      enum PW_Alg al);
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_password>(thd, this); }
 };
 
@@ -999,7 +1040,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("des_encrypt") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_des_encrypt>(thd, this); }
 };
 
@@ -1018,7 +1059,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("des_decrypt") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_des_decrypt>(thd, this); }
 };
 
@@ -1063,7 +1104,7 @@ public:
   {
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_encrypt>(thd, this); }
 };
 
@@ -1087,7 +1128,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("encode") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_encode>(thd, this); }
 protected:
   virtual void crypto_transform(String *);
@@ -1112,7 +1153,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("decode") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_decode>(thd, this); }
 protected:
   void crypto_transform(String *) override;
@@ -1123,20 +1164,20 @@ class Item_func_sysconst :public Item_str_func
 {
 public:
   Item_func_sysconst(THD *thd): Item_str_func(thd)
-  { collation.set(system_charset_info,DERIVATION_SYSCONST); }
-  Item *safe_charset_converter(THD *thd, CHARSET_INFO *tocs);
+  { collation.set(system_charset_info_for_i_s, DERIVATION_SYSCONST); }
+  Item *safe_charset_converter(THD *thd, CHARSET_INFO *tocs) override;
   /*
     Used to create correct Item name in new converted item in
     safe_charset_converter, return string representation of this function
     call
   */
-  virtual const char *fully_qualified_func_name() const = 0;
-  bool check_vcol_func_processor(void *arg)
+  virtual const Lex_ident_routine fully_qualified_func_name() const = 0;
+  bool check_vcol_func_processor(void *arg) override
   {
-    return mark_unsupported_function(fully_qualified_func_name(), arg,
+    return mark_unsupported_function(fully_qualified_func_name().str, arg,
                                      VCOL_SESSION_FUNC);
   }
-  bool const_item() const;
+  bool const_item() const override;
 };
 
 
@@ -1147,7 +1188,7 @@ public:
   String *val_str(String *) override;
   bool fix_length_and_dec(THD *thd) override
   {
-    max_length= NAME_CHAR_LEN * system_charset_info->mbmaxlen;
+    fix_char_length(NAME_CHAR_LEN);
     set_maybe_null();
     return FALSE;
   }
@@ -1156,9 +1197,9 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("database") };
     return name;
   }
-  const char *fully_qualified_func_name() const override
-  { return "database()"; }
-  Item *get_copy(THD *thd) override
+  const Lex_ident_routine fully_qualified_func_name() const override
+  { return Lex_ident_routine("database()"_LEX_CSTRING); }
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_database>(thd, this); }
 };
 
@@ -1173,20 +1214,23 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("SQLERRM") };
     return name;
   }
-  const char *fully_qualified_func_name() const override
-  { return "SQLERRM"; }
+  const Lex_ident_routine fully_qualified_func_name() const override
+  { return Lex_ident_routine("SQLERRM"_LEX_CSTRING); }
   void print(String *str, enum_query_type query_type) override
   {
     str->append(func_name_cstring());
   }
   bool fix_length_and_dec(THD *thd) override
   {
-    max_length= 512 * system_charset_info->mbmaxlen;
+    fix_char_length(512);
     null_value= false;
     base_flags&= ~item_base_t::MAYBE_NULL;
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  // block standard processor for never null
+  bool add_maybe_null_after_ora_join_processor(void *arg) override
+  { return 0; }
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_sqlerrm>(thd, this); }
 };
 
@@ -1199,7 +1243,7 @@ protected:
 public:
   Item_func_user(THD *thd): Item_func_sysconst(thd)
   {
-    str_value.set("", 0, system_charset_info);
+    str_value.set("", 0, collation.collation);
   }
   String *val_str(String *) override
   {
@@ -1218,13 +1262,13 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("user") };
     return name;
   }
-  const char *fully_qualified_func_name() const override
-  { return "user()"; }
+  const Lex_ident_routine fully_qualified_func_name() const override
+  { return Lex_ident_routine("user()"_LEX_CSTRING); }
   int save_in_field(Field *field, bool no_conversions) override
   {
     return save_str_value_in_field(field, &str_value);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_user>(thd, this); }
 };
 
@@ -1242,14 +1286,61 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("current_user") };
     return name;
   }
-  const char *fully_qualified_func_name() const override
-  { return "current_user()"; }
+  const Lex_ident_routine fully_qualified_func_name() const override
+  { return Lex_ident_routine("current_user()"_LEX_CSTRING); }
   bool check_vcol_func_processor(void *arg) override
   {
     context= 0;
-    return mark_unsupported_function(fully_qualified_func_name(), arg,
+    return mark_unsupported_function(fully_qualified_func_name().str, arg,
                                      VCOL_SESSION_FUNC);
   }
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_current_user>(thd, this); }
+};
+
+
+class Item_func_current_path :public Item_func_sysconst
+{
+public:
+  Item_func_current_path(THD *thd): Item_func_sysconst(thd) {}
+  String *val_str(String *) override;
+  bool fix_length_and_dec(THD *thd) override
+  {
+    max_length=32767;
+    return FALSE;
+  }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("current_path") };
+    return name;
+  }
+  const Lex_ident_routine fully_qualified_func_name() const override
+  { return Lex_ident_routine("current_path()"_LEX_CSTRING); }
+  bool check_vcol_func_processor(void *arg) override
+  {
+    return mark_unsupported_function(fully_qualified_func_name().str, arg,
+                                     VCOL_SESSION_FUNC);
+  }
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_current_path>(thd, this); }
+};
+
+
+class Item_func_session_user :public Item_func_user
+{
+public:
+  Item_func_session_user(THD *thd):
+    Item_func_user(thd) {}
+  bool fix_fields(THD *thd, Item **ref) override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("session_user") };
+    return name;
+  }
+  const Lex_ident_routine fully_qualified_func_name() const override
+  { return Lex_ident_routine("session_user()"_LEX_CSTRING); }
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_session_user>(thd, this); }
 };
 
 
@@ -1273,8 +1364,8 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("current_role") };
     return name;
   }
-  const char *fully_qualified_func_name() const override
-  { return "current_role()"; }
+  const Lex_ident_routine fully_qualified_func_name() const override
+  { return Lex_ident_routine("current_role()"_LEX_CSTRING); }
   String *val_str(String *) override
   {
     DBUG_ASSERT(fixed());
@@ -1283,11 +1374,14 @@ public:
   bool check_vcol_func_processor(void *arg) override
   {
     context= 0;
-    return mark_unsupported_function(fully_qualified_func_name(), arg,
+    return mark_unsupported_function(fully_qualified_func_name().str, arg,
                                      VCOL_SESSION_FUNC);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_current_role>(thd, this); }
+  // null do not depend on nullability of the argument
+  bool add_maybe_null_after_ora_join_processor(void *arg) override
+  { return 0; }
 };
 
 
@@ -1303,7 +1397,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("soundex") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_soundex>(thd, this); }
 };
 
@@ -1321,7 +1415,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("elt") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_elt>(thd, this); }
 };
 
@@ -1339,7 +1433,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("make_set") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_make_set>(thd, this); }
 };
 
@@ -1360,7 +1454,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("format") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_format>(thd, this); }
 };
 
@@ -1389,7 +1483,7 @@ public:
     return name;
   }
   void print(String *str, enum_query_type query_type) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_char>(thd, this); }
 };
 
@@ -1409,7 +1503,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("chr") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_chr>(thd, this); }
 };
 
@@ -1426,7 +1520,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("repeat") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_repeat>(thd, this); }
 };
 
@@ -1442,7 +1536,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("space") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_space>(thd, this); }
 };
 
@@ -1463,7 +1557,7 @@ public:
   {
     return mark_unsupported_function(func_name(), "()", arg, VCOL_IMPOSSIBLE);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_binlog_gtid_pos>(thd, this); }
 };
 
@@ -1505,7 +1599,7 @@ public:
     return name;
   }
   Sql_mode_dependency value_depends_on_sql_mode() const override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_rpad>(thd, this); }
 };
 
@@ -1539,7 +1633,7 @@ public:
       print_sql_mode_qualified_name(str, query_type);
     print_args_parenthesized(str, query_type);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_rpad_oracle>(thd, this); }
 };
 
@@ -1565,7 +1659,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("lpad") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_lpad>(thd, this); }
 };
 
@@ -1599,7 +1693,7 @@ public:
       print_sql_mode_qualified_name(str, query_type);
     print_args_parenthesized(str, query_type);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_lpad_oracle>(thd, this); }
 };
 
@@ -1618,11 +1712,11 @@ public:
   bool fix_length_and_dec(THD *thd) override
   {
     collation.set(default_charset());
-    fix_char_length(64);
+    fix_char_length(65);
     set_maybe_null();
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_conv>(thd, this); }
 };
 
@@ -1656,13 +1750,23 @@ public:
   }
   bool fix_length_and_dec(THD *thd) override
   {
+    m_arg0_type_handler= args[0]->type_handler();
     collation.set(default_charset(), DERIVATION_COERCIBLE, MY_REPERTOIRE_ASCII);
     decimals=0;
-    fix_char_length(args[0]->max_length * 2);
-    m_arg0_type_handler= args[0]->type_handler();
+    /*
+      Reserve space for 16 characters for signed numeric data types:
+        hex(-1) -> 'FFFFFFFFFFFFFFFF'.
+      For unsigned numeric types, HEX() can create too large columns.
+      This should be eventually fixed to create minimum possible columns.
+    */
+    const Type_handler_numeric *tn=
+      dynamic_cast<const Type_handler_numeric*>(m_arg0_type_handler);
+    size_t char_length= (tn && !(tn->flags() & UNSIGNED_FLAG)) ?
+                        (size_t) 16 : (size_t) args[0]->max_length * 2;
+    fix_char_length(char_length);
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_hex>(thd, this); }
 };
 
@@ -1688,7 +1792,7 @@ public:
     max_length=(1+args[0]->max_length)/2;
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_unhex>(thd, this); }
 };
 
@@ -1727,7 +1831,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("like_range_min") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_like_range_min>(thd, this); }
 };
 
@@ -1742,7 +1846,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("like_range_max") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_like_range_max>(thd, this); }
 };
 #endif
@@ -1774,7 +1878,7 @@ public:
     return name;
   }
   bool need_parentheses_in_default() override { return true; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_binary>(thd, this); }
 };
 
@@ -1801,7 +1905,7 @@ public:
   {
     return mark_unsupported_function(func_name(), "()", arg, VCOL_IMPOSSIBLE);
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_load_file>(thd, this); }
 };
 
@@ -1822,7 +1926,7 @@ class Item_func_export_set: public Item_str_func
     static LEX_CSTRING name= {STRING_WITH_LEN("export_set") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_export_set>(thd, this); }
 };
 
@@ -1849,7 +1953,7 @@ public:
     max_length= (uint32) MY_MIN(max_result_length, MAX_BLOB_WIDTH);
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_quote>(thd, this); }
 };
 
@@ -1862,13 +1966,13 @@ public:
   Item_func_conv_charset(THD *thd, Item *a, CHARSET_INFO *cs):
     Item_str_func(thd, a)
   {
-    collation.set(cs, DERIVATION_IMPLICIT);
+    collation= DTCollation::string_typecast(cs);
     use_cached_value= 0; safe= 0;
   }
   Item_func_conv_charset(THD *thd, Item *a, CHARSET_INFO *cs, bool cache_if_const):
     Item_str_func(thd, a)
   {
-    collation.set(cs, DERIVATION_IMPLICIT);
+    collation= DTCollation::string_typecast(cs);
     if (cache_if_const && args[0]->can_eval_in_optimize())
     {
       uint errors= 0;
@@ -1945,7 +2049,7 @@ public:
     return name;
   }
   void print(String *str, enum_query_type query_type) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_conv_charset>(thd, this); }
   int save_in_field(Field*, bool) override;
 };
@@ -1953,13 +2057,15 @@ public:
 class Item_func_set_collation :public Item_str_func
 {
   Lex_extended_collation_st m_set_collation;
+  bool check_arguments() const override
+  { return check_argument_types_can_return_str(0, 1); }
 public:
   Item_func_set_collation(THD *thd, Item *a,
                           const Lex_extended_collation_st &set_collation):
     Item_str_func(thd, a), m_set_collation(set_collation) {}
   String *val_str(String *) override;
   bool fix_length_and_dec(THD *thd) override;
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq(const Item *item, const Eq_config &config) const override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("collate") };
@@ -1974,7 +2080,7 @@ public:
     return args[0]->field_for_view_update();
   }
   bool need_parentheses_in_default() override { return true; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_set_collation>(thd, this); }
 };
 
@@ -1985,11 +2091,14 @@ public:
   Item_func_expr_str_metadata(THD *thd, Item *a): Item_str_func(thd, a) { }
   bool fix_length_and_dec(THD *thd) override
   {
-     collation.set(system_charset_info);
+     collation.set(system_charset_info_for_i_s, DERIVATION_SYSCONST);
      max_length= 64 * collation.collation->mbmaxlen; // should be enough
      base_flags&= ~item_base_t::MAYBE_NULL;
      return FALSE;
   };
+  // block standard processor for never null
+  bool add_maybe_null_after_ora_join_processor(void *arg) override
+  { return 0; }
   table_map not_null_tables() const override { return 0; }
   Item* propagate_equal_fields(THD *thd, const Context &ctx, COND_EQUAL *cond)
     override
@@ -2000,6 +2109,8 @@ public:
 
 class Item_func_charset :public Item_func_expr_str_metadata
 {
+  LEX_CSTRING m_cached_charset_info;
+
 public:
   Item_func_charset(THD *thd, Item *a)
     :Item_func_expr_str_metadata(thd, a) { }
@@ -2009,8 +2120,25 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("charset") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_charset>(thd, this); }
+  table_map used_tables() const override { return 0; }
+  bool fix_length_and_dec(THD* thd) override
+  {
+    if (Item_func_expr_str_metadata::fix_length_and_dec(thd))
+      return true;
+    /*
+      Since this is a const item which doesn't use tables (see used_tables()),
+      we don't want to access the function arguments during execution.
+      That's why we store the charset here during the preparation phase
+      and only return it later at the execution phase
+    */
+    DBUG_ASSERT(args[0]->fixed());
+    m_cached_charset_info.str= args[0]->charset_for_protocol()->cs_name.str;
+    m_cached_charset_info.length=
+        args[0]->charset_for_protocol()->cs_name.length;
+    return false;
+  }
 };
 
 
@@ -2025,7 +2153,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("collation") };
     return name;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_collation>(thd, this); }
 };
 
@@ -2052,9 +2180,9 @@ public:
   }
   String *val_str(String *) override;
   bool fix_length_and_dec(THD *thd) override;
-  bool eq(const Item *item, bool binary_cmp) const override
+  bool eq(const Item *item, const Eq_config &config) const override
   {
-    if (!Item_str_func::eq(item, binary_cmp))
+    if (!Item_str_func::eq(item, config))
       return false;
     Item_func_weight_string *that= (Item_func_weight_string *)item;
     return this->weigth_flags == that->weigth_flags &&
@@ -2065,7 +2193,7 @@ public:
     override
   { return this; }
   void print(String *str, enum_query_type query_type) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_weight_string>(thd, this); }
 };
 
@@ -2096,7 +2224,7 @@ public:
   }
   bool fix_length_and_dec(THD *thd) override { max_length=10; return FALSE; }
   longlong val_int() override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_crc32>(thd, this); }
 };
 
@@ -2117,7 +2245,7 @@ public:
     set_maybe_null();
     return FALSE; }
   longlong val_int() override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_uncompressed_length>(thd, this); }
 };
 
@@ -2144,7 +2272,7 @@ public:
     return name;
   }
   String *val_str(String *) override ZLIB_DEPENDED_FUNCTION
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_compress>(thd, this); }
 };
 
@@ -2166,7 +2294,7 @@ public:
     return name;
   }
   String *val_str(String *) override ZLIB_DEPENDED_FUNCTION
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_uncompress>(thd, this); }
 };
 
@@ -2193,7 +2321,7 @@ public:
   String *val_str(String *) override;
   void print(String *str, enum_query_type query_type) override;
   enum Functype functype() const override { return DYNCOL_FUNC; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_dyncol_create>(thd, this); }
 };
 
@@ -2211,7 +2339,7 @@ public:
   }
   String *val_str(String *) override;
   void print(String *str, enum_query_type query_type) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_dyncol_add>(thd, this); }
 };
 
@@ -2233,7 +2361,7 @@ public:
     decimals= 0;
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_dyncol_json>(thd, this); }
 };
 
@@ -2283,7 +2411,7 @@ public:
   bool get_dyn_value(THD *thd, DYNAMIC_COLUMN_VALUE *val, String *tmp);
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
   void print(String *str, enum_query_type query_type) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_dyncol_get>(thd, this); }
 };
 
@@ -2305,7 +2433,7 @@ public:
     return name;
   }
   String *val_str(String *) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_dyncol_list>(thd, this); }
 };
 
@@ -2331,10 +2459,11 @@ public:
     return name;
   }
   bool fix_length_and_dec(THD *thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_temptable_rowid>(thd, this); }
 };
 
+
 class Item_func_format_pico_time : public Item_str_ascii_func
 {
   /* Format is 'AAAA.BB UUU' = 11 characters or 'AAA ps' = 6 characters. */
@@ -2355,12 +2484,39 @@ public:
     fix_length_and_charset(sizeof(m_value_buffer), default_charset());
     return false;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   {
     return get_item_copy<Item_func_format_pico_time>(thd, this);
   }
 };
 
+
+class Item_func_format_bytes : public Item_str_ascii_func
+{
+  /* Format is '-A.AAe+BB UUU' = 13 or 'AAAA.BB UUU' = 11 characters or 'AAAA bytes' = 10 characters. */
+  char m_value_buffer[14];
+  String m_value;
+
+public:
+    Item_func_format_bytes(THD *thd, Item *a): Item_str_ascii_func(thd, a) {}
+  String *val_str_ascii(String *) override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("format_bytes")};
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  {
+    m_value.set(m_value_buffer, sizeof(m_value_buffer), default_charset());
+    fix_length_and_charset(sizeof(m_value_buffer), default_charset());
+    return false;
+  }
+  Item *do_get_copy(THD *thd) const override
+  {
+    return get_item_copy<Item_func_format_bytes>(thd, this);
+  }
+};
+
 #ifdef WITH_WSREP
 
 #include "wsrep_api.h"
@@ -2382,7 +2538,7 @@ public:
     set_maybe_null();
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_wsrep_last_written_gtid>(thd, this); }
 };
 
@@ -2403,7 +2559,7 @@ public:
     set_maybe_null();
     return FALSE;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_wsrep_last_seen_gtid>(thd, this); }
 };
 
@@ -2421,7 +2577,7 @@ public:
     return name;
   }
   longlong val_int() override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_wsrep_sync_wait_upto>(thd, this); }
 };
 #endif /* WITH_WSREP */
